/*
	Tooltip DLL v0.1
	by ClickHeRe
	panodude@videotron.ca

	This DLL lets the user place some info tip bubbles on dialog controls in mIRC

*/
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#define _WIN32_IE 0x0500 
#include <commctrl.h>
#include <stdlib.h>

// alias to define an External DLL Command
#define mIRC(x) int __stdcall WINAPI x(HWND mWnd,HWND aWnd,char *data,char*,BOOL,BOOL)
// alias to define a WNDPROC function
#define Proc(x) LRESULT CALLBACK x(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)

#define NBMAX 255
#define CMAX 128

// ##################################################################################################

// Global Vars
HANDLE hFileMap;
LPSTR mData;

// ##################################################################################################
// structures
struct TOOLTIP {
	UINT ToolID;
	HWND hwnd;
	TOOLINFO toolinfo;
	char text[1000];
	char title[1000];
	int icon;
	int width;
	int active;
  COLORREF text_color;
	COLORREF bkg_color;
};
struct WINSTRUCT
{
	HWND Whwnd;
	WNDPROC OldProc;
	struct TOOLTIP tooltip[CMAX];
}; 
struct WINSTRUCT Window[NBMAX];

// default Load DLL structure
typedef struct {
    DWORD  mVersion;
    HWND   mHwnd;
    BOOL   mKeep;
} LOADINFO;

// ##################################################################################################
// Freeing ressources
void DeleteTooltip(int i,int j) {
	// destroy Tooltip Window
	// DestroyWindow(Window[i].tooltip[j].hwnd);
	// Reset tooltip hwnds for another dialog
  Window[i].tooltip[j].hwnd = 0;
	Window[i].tooltip[j].ToolID = 0;
}
void FreeWinRessources(int i) {
	// give back old winproc to dialog
  SetWindowLong(Window[i].Whwnd,GWL_WNDPROC,(LONG)Window[i].OldProc);
	// reset window HWND for other mark process
  Window[i].Whwnd = 0;
	// delete all tooltips used inside marked window
  int j = 0;
  while (j <= CMAX) {
		if (Window[i].tooltip[j].hwnd) DeleteTooltip(i,j);
		j++;
	}
}

// ##################################################################################################
// Load DLL command
void WINAPI LoadDll(LOADINFO* load)
{
	hFileMap = CreateFileMapping(INVALID_HANDLE_VALUE,0,PAGE_READWRITE,0,4096,"mIRC");     
  mData = (LPSTR)MapViewOfFile(hFileMap,FILE_MAP_ALL_ACCESS,0,0,0);
   
	// Init common controls
	INITCOMMONCONTROLSEX iccex; 
	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	iccex.dwICC = ICC_WIN95_CLASSES;
	InitCommonControlsEx(&iccex);
}

// Unload DLL command
int WINAPI UnloadDll(int timeout)
{
	if (!timeout)
  { 

		int i=0,j=0;
		while (i < NBMAX) {
			if (Window[i].Whwnd) FreeWinRessources(i);		   
			i++;
		}
		UnmapViewOfFile(mData);
		CloseHandle(hFileMap);
		return 1;
	}
  else return 0;
}

// ##################################################################################################
// Dialog Procedure Callback
Proc(DialogProc)
{
	int i = 0,j = 0;
	// find which window was handle
	while(Window[i].Whwnd != hwnd) i++;

	switch(uMsg)
	{
		// free ressources assciated with window
		case WM_DESTROY:			
			FreeWinRessources(i);
			break;
	}
	return CallWindowProc(Window[i].OldProc,hwnd,uMsg,wParam,lParam);
}

// ##################################################################################################
// External DLL Commands
mIRC(Version) {
	lstrcpy(data,"ToolTip DLL v0.1 by ClickHeRe 2003. Member of the SDB.org Team http://scriptsdb.org");
	return 3;
}

mIRC(Mark) {

	int i = 0;
	HWND hwnd = (HWND) atoi(data);
	// check if window handle exists
	if (!IsWindow(hwnd)) {
	 wsprintf(data,"ERROR Mark: invalid window");
	 return 3;
	}
	// find if window was already marked
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++;
	}  

  if (i != NBMAX) {
		wsprintf(data,"ERROR Mark: window is already marked");
		return 3;
	}
	// find first non used ressource window
	i = 0;
	while (Window[i].Whwnd && i < NBMAX) i++;
	if (i == NBMAX) {
		wsprintf(data,"ERROR Mark: %d windows are already marked.",NBMAX);
		return 3;
	}
	// Mark Window
	Window[i].Whwnd = hwnd;
	// Override Old Window Procedure
	Window[i].OldProc = (WNDPROC)SetWindowLong(Window[i].Whwnd,GWL_WNDPROC,(LONG)DialogProc);
  wsprintf(data,"S_OK window marked.");
	return 3;
}

mIRC(ToolTip) {

	HWND hwnd, hwndTT;
	UINT ID; 
	char chwnd[20];
	char cID[10];
	char params[500];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(params,"%s",strtok(NULL," "));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR Tooltip: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR Tooltip: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR Tooltip: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) { 
		  wsprintf(data,"ERROR Tooltip: Dialog ID %d Has Already it's Tooltip",ID - 6000);
			return 3;
		}
		j++;
	}

	// find the first empty Tooltip on Window[i]
	j = 0;
	while (j < CMAX) {
    if (!Window[i].tooltip[j].hwnd) { break; }
		j++;
	}

	if (i == CMAX) {
		wsprintf(data,"ERROR Tooltip: Dialog Already Has %d Max Tooltips Activated.",CMAX);
		return 3;
	}

  int styles = WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP;

	if (!lstrcmp(params,"balloon")) styles |= TTS_BALLOON;

	hwndTT = CreateWindowEx(WS_EX_TOPMOST,
    TOOLTIPS_CLASS,
    NULL,
		styles,
    //WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP, 
		//| TTS_BALLOON,		
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    hwnd,
    NULL,
    NULL,
    NULL
    );

	if (!IsWindow(hwndTT)) {
    wsprintf(data,"ERROR Tooltip: Unable to Create Tooltip Window");
		return 3;
	}
	
	// setup window tooltip infos
	Window[i].tooltip[j].ToolID = ID;
	Window[i].tooltip[j].hwnd = hwndTT;

	wsprintf(Window[i].tooltip[j].title,"");
	wsprintf(Window[i].tooltip[j].text,"");

	Window[i].tooltip[j].toolinfo.cbSize = sizeof(TOOLINFO);
	Window[i].tooltip[j].toolinfo.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
  Window[i].tooltip[j].toolinfo.hwnd = hwnd;
  Window[i].tooltip[j].toolinfo.hinst = NULL;
  Window[i].tooltip[j].toolinfo.uId = (UINT) GetDlgItem(hwnd,ID);
  Window[i].tooltip[j].toolinfo.lpszText = Window[i].tooltip[j].text;

	// add tooltip to dialog ID
	SendMessage(hwndTT, TTM_ADDTOOL, 0, (LPARAM) &Window[i].tooltip[j].toolinfo );
	// activate tooltip
	SendMessage(hwndTT, TTM_ACTIVATE, (WPARAM) TRUE, 0 );
	// set maximum tooltip window width
	SendMessage(hwndTT, TTM_SETMAXTIPWIDTH, 0, 250);

	wsprintf(data,"S_OK: Tooltip Created On ID %d",ID - 6000);
	return 3;
}

mIRC(SetText) { 

	HWND hwnd;
	UINT ID; 
	char chwnd[20];
	char cID[10];
	char params[1000];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(params,"%s",strtok(NULL,""));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR SetText: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR SetText: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR SetText: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) break;
		else j++;
	}
	// we've reached the max tooltips per dialog window
	if (i == CMAX) {
	  wsprintf(data,"ERROR SetText: Dialog ID has no Tooltip Created, Use Tooltip first");
	  return 3;
  }

	Window[i].tooltip[j].toolinfo.lpszText = (LPTSTR) params;
	SendMessage(Window[i].tooltip[j].hwnd, TTM_ADDTOOL, 0, (LPARAM) &Window[i].tooltip[j].toolinfo);

  wsprintf(data,"S_OK: Tooltip Text On ID %d Changed",ID - 6000);
	return 3;
}

mIRC(SetTitle) { 

	HWND hwnd;
	UINT ID; 
	int icon = 0;
	char chwnd[20];
	char cID[10];
	char cicon[2];
	char title[1000];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(cicon,"%s",strtok(NULL," "));
	wsprintf(title,"%s",strtok(NULL,""));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);
	icon = (int) atoi(cicon);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR SetTitle: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR SetTitle: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR SetTitle: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) break;
		else j++;
	}
	
	if (i == CMAX) {
	  wsprintf(data,"ERROR SetTitle: Dialog ID has no Tooltip Created, Use Tooltip first");
	  return 3;
  }

	Window[i].tooltip[j].icon = icon;
  wsprintf(Window[i].tooltip[j].title,title);
	SendMessage(Window[i].tooltip[j].hwnd, TTM_SETTITLE, icon, (LPARAM) title);

  wsprintf(data,"S_OK: Tooltip Title On ID %d Changed",ID - 6000);
	return 3;
}

mIRC(SetTxtColor) {
	
	HWND hwnd;
	UINT ID; 
	COLORREF color;
	char chwnd[20];
	char cID[10];
	char ccolor[40];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(ccolor,"%s",strtok(NULL," "));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);
	color = (COLORREF) atoi(ccolor);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR SetTextColor: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR SetTextColor: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR SetTextColor: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) break;
		else j++;
	}
	
	if (i == CMAX) {
	  wsprintf(data,"ERROR SetTextColor: Dialog ID has no Tooltip Created, Use Tooltip first");
	  return 3;
  }

	Window[i].tooltip[j].text_color = color;
	SendMessage(Window[i].tooltip[j].hwnd, TTM_SETTIPTEXTCOLOR, (WPARAM) color, 0);
	wsprintf(data,"S_OK: Tooltip Text Color On ID %d Changed",ID - 6000);
	return 3;
}


mIRC(SetBkgColor) { 

	HWND hwnd;
	UINT ID; 
	COLORREF color;
	char chwnd[20];
	char cID[10];
	char ccolor[40];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(ccolor,"%s",strtok(NULL," "));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);
	color = (COLORREF) atoi(ccolor);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR SetBkgColor: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR SetBkgColor: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR SetBkgColor: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) break;
		else j++;
	}
	
	if (i == CMAX) {
	  wsprintf(data,"ERROR SetBkgColor: Dialog ID has no Tooltip Created, Use Tooltip first");
	  return 3;
  }

	Window[i].tooltip[j].bkg_color = color;
	SendMessage(Window[i].tooltip[j].hwnd, TTM_SETTIPBKCOLOR, (WPARAM) color, 0);

	wsprintf(data,"S_OK: Tooltip Bkg Color On ID %d Changed",ID - 6000);
	return 3;
}

mIRC(SetWidth) {

	HWND hwnd;
	UINT ID; 
	int width;
	char chwnd[20];
	char cID[10];
	char cwidth[40];
	// extract window hwnd
	wsprintf(chwnd,"%s",strtok(data,">"));
	// extract Control ID
	wsprintf(cID,"%s",strtok(NULL," "));
	// extract Tooltip parameters
	wsprintf(cwidth,"%s",strtok(NULL," "));
	// convert hwnd & ID to numbers
  hwnd = (HWND) atoi(chwnd);
	ID = 6000 + (UINT) atoi(cID);
	width = (int) atoi(cwidth);

  // check if window exists
	if (!IsWindow(hwnd)) { 
		lstrcpy(data,"ERROR SetWidth: Invalid Dialog Window Handle");
		return 3;
	}
	// check if Dialog ID exists
	if (!IsWindow(GetDlgItem(hwnd,ID))) {
    lstrcpy(data,"ERROR SetWidth: Invalid Dialog Control ID");
		return 3;
	}

	// find if window was marked before attempting to use it
	int i = 0;
	while (i < NBMAX) {
	  if (Window[i].Whwnd == hwnd) break;
	  else i++; 
  }  
  if (i == NBMAX) {
	  wsprintf(data,"ERROR SetWidth: window must be marked first");
	  return 3;
  }

	// find if Dialog ID has already it's tooltip
	int j = 0;
  while (j < CMAX) {
		if (Window[i].tooltip[j].ToolID == ID) break;
		else j++;
	}
	
	if (i == CMAX) {
	  wsprintf(data,"ERROR SetWidth: Dialog ID has no Tooltip Created, Use Tooltip first");
	  return 3;
  }

	Window[i].tooltip[j].width = width;
	SendMessage(Window[i].tooltip[j].hwnd, TTM_SETMAXTIPWIDTH, 0, width);

	wsprintf(data,"S_OK: Tooltip Width On ID %d Changed",ID - 6000);
	return 3;
}